//******************************************************************
//*******************************************************************
//*** Client
//*******************************************************************
//*******************************************************************

//-------------------------------------------------------------------
//  Required definitions
//-------------------------------------------------------------------
#define UNICODE
#define _WIN32_DCOM

//-------------------------------------------------------------------
//  includes
//-------------------------------------------------------------------
#include <assert.h>
#include <stdio.h>
#include <windows.h>
#include <initguid.h>
#include "..\..\..\basics\OCREngine\idl\ocr.h"

//*******************************************************************
//*******************************************************************
//***  UTILITY FUNCTIONS
//*******************************************************************
//*******************************************************************
void DisplayStatus(wchar_t *pwszMsg, HRESULT hr)
{

   if (hr == S_OK)
   {
      wprintf(TEXT("%s\n"), pwszMsg);
      return;
   }

   if (HRESULT_FACILITY(hr) == FACILITY_WINDOWS)
      hr = HRESULT_CODE(hr);

   wchar_t *pwszStatus;
   FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER |
      FORMAT_MESSAGE_FROM_SYSTEM,
      NULL,
      hr,
      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
      (LPWSTR)&pwszStatus,
      0,
      NULL );

   wprintf(TEXT("%s: %s (ECode: %lx)\n"), pwszMsg, pwszStatus, hr);

   LocalFree(pwszStatus);
}

//*******************************************************************
//*******************************************************************
//*** Main
//*******************************************************************
//*******************************************************************
void main(int argc, char **argv)
{
   DisplayStatus(TEXT("Client: Started"), S_OK);
   HRESULT hr = S_OK;
   hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
   assert(SUCCEEDED(hr));

/*
   {
      MULTI_QI mqi[] = { {&IID_IOcr, NULL, S_OK} };
      hr = CoCreateInstanceEx(CLSID_OcrEngine, 
                        NULL, 
                        CLSCTX_SERVER, 
                        NULL, 
                        sizeof(mqi)/sizeof(mqi[0]), 
                        mqi);
      DisplayStatus(TEXT("CoCreateInstanceEx..."), hr);
      assert(SUCCEEDED(hr));

      char *pbImage = "0000";
      wchar_t *pOut=0;
      IOcr *pOcr = static_cast<IOcr *>(mqi[0].pItf);
      hr = pOcr->OcrImage(4, (byte*)pbImage, &pOut);
      DisplayStatus(TEXT("IOcr::OcrImage"), hr);
      assert(SUCCEEDED(hr));
      CoTaskMemFree(pOut);
      pOcr->Release();

   }
*/
   {
      IClassFactory *pFactory=0;
      hr = CoGetClassObject(CLSID_OcrEngine, CLSCTX_SERVER, 0, IID_IClassFactory, (void**)&pFactory);
      DisplayStatus(TEXT("CoGetClassObject..."), hr); assert(SUCCEEDED(hr));
      IOcr *pOcr =0;
      hr = pFactory->CreateInstance(NULL, IID_IOcr, reinterpret_cast<void**>(&pOcr));
      DisplayStatus(TEXT("pFactory->CreateInstance..."), hr);
      assert(SUCCEEDED(hr));

      pFactory->Release();
      char *pbImage = "0000"; wchar_t *pOut=0;
      hr = pOcr->OcrImage(4, (byte*)pbImage, &pOut);
      DisplayStatus(TEXT("IOcr::OcrImage"), hr);
      assert(SUCCEEDED(hr));
      CoTaskMemFree(pOut);
      pOcr->Release();
   }

   CoUninitialize();
   DisplayStatus(TEXT("Client shutting down..."), S_OK);
}

