//*******************************************************************
//*******************************************************************
//***  Implementation of the CoDetails component object
//***  This component is an outofproc component that is used for
//***  memory management and performance exercises.
//*******************************************************************
//*******************************************************************

//-------------------------------------------------------------------
// Required macros
//-------------------------------------------------------------------
#define UNICODE         // UNICODE
#define _WIN32_DCOM     // DCOM

//-------------------------------------------------------------------
// includes
//-------------------------------------------------------------------
#include <assert.h>
#include <stdio.h>
#include <windows.h>
#include <initguid.h>
#include "..\idl\details.h"

//-------------------------------------------------------------------
// global event telling use wen to shutdown our application
//-------------------------------------------------------------------
HANDLE g_hExitEvent;

//-------------------------------------------------------------------
// Component Level Reference count for the lifetime managment of 
// the whole component.
// These specific implementations are used for out-of-process servers
// There are a pair of functions like these that are used for
// in-process servers.
//-------------------------------------------------------------------
inline ULONG ComponentAddRef()
{
   ULONG ul = CoAddRefServerProcess();
   wprintf(TEXT("ComponentAddRef(%ld)\n"), ul);
   return ul ;
}

inline ULONG ComponentRelease()
{
   ULONG ul = CoReleaseServerProcess();

   // wait 3 second to test for REGCLS_MULTIPLEUSE
   // Sleep(3000);

   wprintf(TEXT("ComponentRelease(%ld)\n"), ul);
   if (ul==0) {
      SetEvent(g_hExitEvent);
   }
   return ul ;
}


//*******************************************************************
//*******************************************************************
//***  UTILITY FUNCTIONS
//*******************************************************************
//*******************************************************************
void DisplayStatus(wchar_t *pwszMsg, HRESULT hr)
{

   if (hr == S_OK) {
      wprintf(TEXT("%s\n"), pwszMsg);
      return;
   }

   if (HRESULT_FACILITY(hr) == FACILITY_WINDOWS) {
      hr = HRESULT_CODE(hr);
   }

   wchar_t *pwszStatus;
   FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER |
       FORMAT_MESSAGE_FROM_SYSTEM,
       NULL,
       hr,
       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
       (LPWSTR)&pwszStatus,
       0,
       NULL );

   wprintf(TEXT("%s: %s (ECode: %lx)\n"), pwszMsg, pwszStatus, hr);

   LocalFree(pwszStatus);
}

//-------------------------------------------------------------------
// Given a unicode string, get the number of bytes needed to host it
//-------------------------------------------------------------------
inline long ByteLen(wchar_t *pwsz) 
{
   return (sizeof(wchar_t)*(wcslen(pwsz)+1));
}

//-------------------------------------------------------------------
// Automatic registration; for out-of-process server support the
// -regserver option
//-------------------------------------------------------------------
void RegisterComponent()
{
   wchar_t wszKey[MAX_PATH];
   wchar_t wszValue[MAX_PATH];
   HKEY hKey = 0;

   // HKEY_CLASSES_ROOT\CLSID\{3CC346E1-D3CC-11d1-915C-006008052F2D}
   //  @="Details"
   wcscpy(wszKey, TEXT("CLSID\\{3CC346E1-D3CC-11d1-915C-006008052F2D}"));
   RegCreateKey(HKEY_CLASSES_ROOT, wszKey, &hKey);
   wcscpy(wszValue, TEXT("Details"));
   RegSetValueEx(hKey, 0, 0, REG_SZ, (BYTE*)wszValue, ByteLen(wszValue));

   //  "AppID"="{B475EB51-D3CD-11d1-915C-006008052F2D}"
   wcscpy(wszValue, TEXT("{B475EB51-D3CD-11d1-915C-006008052F2D}"));
   RegSetValueEx(hKey, TEXT("AppID"), 0, REG_SZ, 
                 (BYTE*)wszValue, ByteLen(wszValue));
   RegCloseKey(hKey);

   // HKEY_CLASSES_ROOT\CLSID\{3CC346E1-D3CC-11d1-915C-006008052F2D}\LocalServer32
   //      @="...path...\detailsEXE.exe"
   wcscpy(wszKey, TEXT("CLSID\\{3CC346E1-D3CC-11d1-915C-006008052F2D}\\")
          TEXT("LocalServer32"));
   RegCreateKey(HKEY_CLASSES_ROOT, wszKey, &hKey);
   GetModuleFileName(0, wszValue, MAX_PATH);
   RegSetValueEx(hKey, 0, 0, REG_SZ, (BYTE*)wszValue, ByteLen(wszValue));
   RegCloseKey(hKey);

   // HKEY_CLASSES_ROOT\AppID\detailsEXE.exe
   //      "AppID"="{B475EB51-D3CD-11d1-915C-006008052F2D}"
   wcscpy(wszKey, TEXT("AppID\\detailsEXE.exe"));
   RegCreateKey(HKEY_CLASSES_ROOT, wszKey, &hKey);
   wcscpy(wszValue, TEXT("{B475EB51-D3CD-11d1-915C-006008052F2D}"));
   RegSetValueEx(hKey, TEXT("AppID"), 0, REG_SZ, 
                 (BYTE*)wszValue, ByteLen(wszValue));
   RegCloseKey(hKey);

   // HKEY_CLASSES_ROOT\AppID\{B475EB51-D3CD-11d1-915C-006008052F2D}
   //      @="Details"
   wcscpy(wszKey, TEXT("AppID\\{B475EB51-D3CD-11d1-915C-006008052F2D}"));
   RegCreateKey(HKEY_CLASSES_ROOT, wszKey, &hKey);
   wcscpy(wszValue, TEXT("Details"));
   RegSetValueEx(hKey, 0, 0, REG_SZ, 
                 (BYTE*)wszValue, ByteLen(wszValue));
   RegCloseKey(hKey);
}

//-------------------------------------------------------------------
// Automatic unregistration; for out-of-process server support the
// -unregserver option
//-------------------------------------------------------------------
void UnregisterComponent()
{   
   long lRc = 0 ;
   lRc = RegDeleteKey(HKEY_CLASSES_ROOT, TEXT("CLSID\\") 
            TEXT("{3CC346E1-D3CC-11d1-915C-006008052F2D}\\") 
            TEXT("LocalServer32"));
   DisplayStatus(TEXT("Unregistered LocalServer32"), lRc);
   lRc = RegDeleteKey(HKEY_CLASSES_ROOT, TEXT("CLSID\\") 
            TEXT("{3CC346E1-D3CC-11d1-915C-006008052F2D}"));
   DisplayStatus(TEXT("Unregistered CLSID"), lRc);
   lRc = RegDeleteKey(HKEY_CLASSES_ROOT, TEXT("AppID\\") 
            TEXT("{B475EB51-D3CD-11d1-915C-006008052F2D}"));
   DisplayStatus(TEXT("Unregistered AppID"), lRc);
   lRc = RegDeleteKey(HKEY_CLASSES_ROOT, TEXT("AppID\\detailsEXE.exe"));
   DisplayStatus(TEXT("Unregistered detailsEXE.exe"), lRc);
}
 
//*******************************************************************
//*******************************************************************
//***  Class CoDetails - A component object that implements 
//***  interface:  IDetails, IMemoryManagement
//***  Simple implementation that doesn't support aggregation.
//***  This object cannot be aggregated by another object,
//***  Containment works with every component object!
//*******************************************************************
//*******************************************************************
class CoDetails : public IDetails , public IMemoryManagement {
public:
   // constructors/destructors
   CoDetails() : m_lRefCount(0) { ComponentAddRef(); }
   ~CoDetails() { ComponentRelease(); }

   // The factory will call this method to actually 
   // create this object
   static HRESULT CreateObject(LPUNKNOWN pUnkOuter, REFIID riid, 
                               void** ppv);
public: 
   // IUnknown Methods
   STDMETHODIMP QueryInterface(REFIID riid, void **ppv);
   STDMETHODIMP_(ULONG) AddRef(void)
   {
     //------debug messages
       // wchar_t wszMsg[MAX_PATH];
       // wsprintf(wszMsg, TEXT("CoDetails::AddRef (%ld)"), m_lRefCount); 
       // DisplayStatus(wszMsg, S_OK);
       return InterlockedIncrement(&m_lRefCount);
   }
   STDMETHODIMP_(ULONG) Release(void)
   {
     //------debug messages
       // wchar_t wszMsg[MAX_PATH];
       // wsprintf(wszMsg, TEXT("CoDetails::Release (%ld)"), m_lRefCount-1); 
       // DisplayStatus(wszMsg, S_OK);

       long lCount = InterlockedDecrement(&m_lRefCount);
       if (lCount == 0) {
           // DisplayStatus(TEXT("CoDetails::Release - delete object"), S_OK);
           delete this; 
       } 
       return lCount; 
   }

   // IDetails Methods
   STDMETHODIMP VoidCall(void);
   STDMETHODIMP LongCall(long lNumber);
   STDMETHODIMP ArrayCall(long lSize, long *pArrayLong);

   // IMemoryManagement
   STDMETHODIMP OcrZoneAndGetImageDimension(long lImageSize,
           byte *pbImage, struct Zone *pZoneInputDimensionOutput,
           wchar_t **pwszOcrText)
   {
       // use pbImage, its length is lImageSize
       // WriteFile(hFile, static_cast<LPCVOID>(pbImage), lImageSize, &dwWritten, NULL);

       // use the passed in input dimension
       wprintf(TEXT("pZoneInputDimensionOutput = %d\n"), pZoneInputDimensionOutput->m_sX);
       // return new value to caller
       pZoneInputDimensionOutput->m_sX=100;

       // allocate memory and return fake ocr data
       wchar_t wszOcrText[MAX_PATH];
       wcscpy(wszOcrText, TEXT("This is fake OCR text"));
       // remember that we have to take the len of string including null
       // multiply by the sizeof wchar_t
       long lAllocSize = (wcslen(wszOcrText)+1) * sizeof(wchar_t);
       *pwszOcrText = static_cast<wchar_t *>(CoTaskMemAlloc(lAllocSize));
       if (*pwszOcrText==NULL) { return E_OUTOFMEMORY; }
       wcscpy(*pwszOcrText, wszOcrText);

       wprintf(TEXT("Returning = %s\n"), *pwszOcrText);

       return S_OK ;
   }
       
   STDMETHODIMP Pixels2Inches(struct Resolution res,
           long lArraySize, struct Zone *pZone)
   {
     long i=0;
       wprintf(TEXT("Pixels2Inches\n"));
       // dump the values out
     for (i=0; i<lArraySize; i++) {
          wprintf(TEXT("[%d,%d,%d,%d] "), pZone[i].m_sX, 
             pZone[i].m_sY, pZone[i].m_sWidth, pZone[i].m_sHeight);
     }
       wprintf(TEXT("\nsize=%d\tpZone[0] = %d\tpZone[last] = %d\n"), 
          lArraySize, pZone[0].m_sX, pZone[lArraySize-1].m_sX);

       // change all values to 2
     for (i=0; i<lArraySize; i++) {
          pZone[i].m_sX+=1; pZone[i].m_sY+=1; pZone[i].m_sWidth+=1; pZone[i].m_sHeight+=1;
     }

       return S_OK;
   }

  // age is simple
   STDMETHODIMP SetAge(short age)
   { return S_OK; }        
   STDMETHODIMP GetAge(short *age)
   { *age = 100; return S_OK; }        
   STDMETHODIMP ChangeAge(short *age)
   { *age=100; return S_OK; }

   STDMETHODIMP SetBinaryData(BinaryData data)
   {
       wprintf(TEXT("SetBinaryData\n"));

       // use the data passed in
       if (data.m_pBuffer) {
           char szBuf[MAX_PATH]; memset(szBuf, 0, MAX_PATH);
           memcpy(szBuf, data.m_pBuffer, (data.m_lSize>MAX_PATH)?MAX_PATH:data.m_lSize);
           printf("Data=%s\tSize=%d\n", szBuf, data.m_lSize);
       }

       return S_OK; 
   }
        
   STDMETHODIMP GetBinaryData(BinaryData *pData)
   {
      wprintf(TEXT("GetBinaryData\n"));

      char *pBuffer = "HumptyDumpty";
      pData->m_lSize = strlen(pBuffer);
      // callee allocates embedded pointer to return to client
      pData->m_pBuffer = (BYTE*)CoTaskMemAlloc(pData->m_lSize);
      memcpy(pData->m_pBuffer, pBuffer, pData->m_lSize);

      return S_OK; 
   }
        
   STDMETHODIMP ChangeBinaryData(BinaryData *pData)
   {
      wprintf(TEXT("ChangeBinaryData\n"));

      // delete old embedded pointer
      if (pData->m_pBuffer) {
          // use data first
          char szBuf[MAX_PATH]; memset(szBuf, 0, MAX_PATH);
          memcpy(szBuf, pData->m_pBuffer, (pData->m_lSize>MAX_PATH)?MAX_PATH:pData->m_lSize);
          printf("Data=%s\tSize=%d\n", szBuf, pData->m_lSize);
          CoTaskMemFree(pData->m_pBuffer);
      }
      // reallocate new embedded pointer for return to caller
      char *pBuffer = "HumptyHasBeenChanged";
      pData->m_lSize = strlen(pBuffer);
      // callee allocates embedded pointer to return to client
      pData->m_pBuffer = (BYTE*)CoTaskMemAlloc(pData->m_lSize);
      memcpy(pData->m_pBuffer, pBuffer, pData->m_lSize);

      return S_OK; 
   }

private:
    LONG m_lRefCount;
};

//-------------------------------------------------------------------
// CoDetails::CreateObject - static function to create an
// CoDetails object
//-------------------------------------------------------------------
HRESULT CoDetails::CreateObject(LPUNKNOWN pUnkOuter, 
                          REFIID riid, 
                                  void** ppv)
{
   *ppv = NULL;

   // CoDetails doesn't support aggregation
   if (pUnkOuter != NULL) { return CLASS_E_NOAGGREGATION; }

   CoDetails * pDetails = new CoDetails;
   if (pDetails == NULL) { return E_OUTOFMEMORY; }

   HRESULT hr = pDetails->QueryInterface(riid, ppv);
   if (FAILED(hr)) { delete pDetails; }

   return hr;
}

//-------------------------------------------------------------------
// CoDetails::QueryInterface
//-------------------------------------------------------------------
STDMETHODIMP
CoDetails::QueryInterface(REFIID riid, void** ppv)
{
   //DisplayStatus(TEXT("CoDetails::QueryInterface"), S_OK);

   if (ppv==NULL) { return E_INVALIDARG; }

   if (riid==IID_IUnknown) {
      *ppv= static_cast<IDetails *>(this);
   } else if (riid==IID_IDetails) {
      *ppv= static_cast<IDetails *>(this);
   } else if (riid==IID_IMemoryManagement) {
      *ppv= static_cast<IMemoryManagement *>(this);
   } else {
      *ppv=NULL; return E_NOINTERFACE ;
   }

   reinterpret_cast<IUnknown *>(*ppv)->AddRef();

   return S_OK;
}

//-------------------------------------------------------------------
// CoDetails::VoidCall  
// For performance testing....
//-------------------------------------------------------------------
STDMETHODIMP CoDetails::VoidCall(void)
{
   return S_OK;
}

//-------------------------------------------------------------------
// CoDetails::LongCall
// For performance testing....
//-------------------------------------------------------------------
STDMETHODIMP CoDetails::LongCall(long lNumber)
{
   return S_OK;
}

//-------------------------------------------------------------------
// CoDetails::ArrayCall
// For performance testing....
//-------------------------------------------------------------------
STDMETHODIMP CoDetails::ArrayCall(long lSize, long *pArrayLong)
{
   return S_OK;
}

//*******************************************************************
//*******************************************************************
//***  Class Factory: manufacturing CoDetails objects
//*******************************************************************
//*******************************************************************
class CoDetailsFactory : public IClassFactory 
{
public:
   // IUnknown Methods
   STDMETHODIMP QueryInterface (REFIID riid, void** ppv);
   STDMETHODIMP_(ULONG) AddRef(void)
   { return 1; }
   STDMETHODIMP_(ULONG) Release(void) 
   { return 1; }

   // IClassFactory Methods
   STDMETHODIMP CreateInstance(LPUNKNOWN pUnkOuter, 
                               REFIID riid, 
                               void **ppv)
   {
      // call CoDetails's static function to create
      // a CoDetails component object
      return CoDetails::CreateObject(pUnkOuter, riid, ppv);
   }

   STDMETHODIMP LockServer(BOOL fLock)
   {
      if (fLock) {
         ComponentAddRef();
      } else {
         ComponentRelease();
      }
      return S_OK;        
   }  
};

//-------------------------------------------------------------------
//  Singleton factory instance that manufacture CoCorDetails
//  component objects
//-------------------------------------------------------------------
CoDetailsFactory g_DetailsClassFactory;

//-------------------------------------------------------------------
//  CoDetailsFactory::QueryInterface
//-------------------------------------------------------------------
STDMETHODIMP 
CoDetailsFactory::QueryInterface(REFIID riid, void** ppv)
{
   if (ppv==NULL) { return E_INVALIDARG; }

   if (riid==IID_IUnknown) {
      *ppv= static_cast<IClassFactory *>(this);
   } else if (riid==IID_IClassFactory) {
      *ppv= static_cast<IClassFactory *>(this);
   } else {
      *ppv=NULL; return E_NOINTERFACE ;
   }

   reinterpret_cast<IUnknown *>(*ppv)->AddRef();

   return S_OK;
}

//*******************************************************************
//*******************************************************************
//***  Main Program
//*******************************************************************
//*******************************************************************
void main(int argc, char **argv)
{
   DisplayStatus(TEXT("Server: Started"), S_OK);

   g_hExitEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
   assert(g_hExitEvent);

   // registration if required
   if (argc > 1) {
      if (_stricmp(argv[1], "-RegServer")==0){
         RegisterComponent();
         DisplayStatus(TEXT("Registered..."), S_OK);
         return ;
      }
      if (_stricmp(argv[1], "-UnRegServer")==0){
         UnregisterComponent();
         DisplayStatus(TEXT("Unregistered..."), S_OK);
         return ;
      }
   }

   // Init COM
   HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
   assert(SUCCEEDED(hr));

   // Register Details Class Factory with COM
   DWORD dwRegister;
   hr = CoRegisterClassObject(CLSID_Details, 
                              &g_DetailsClassFactory,
                              CLSCTX_SERVER, 
                              REGCLS_MULTIPLEUSE, 
                              &dwRegister);
   assert(SUCCEEDED(hr));
   // we don't have to do this, but it's a good habit
   g_DetailsClassFactory.Release();  

   // sit and wait until CoDetails object is used
   WaitForSingleObject(g_hExitEvent, INFINITE);

   // revoke the factory from public view
   CoRevokeClassObject(dwRegister);

   // uninitialize
   CoUninitialize();

   DisplayStatus(TEXT("Server shutting down in 5 seconds..."), S_OK);

   Sleep(5000);
}
