//******************************************************************
//*******************************************************************
//*** Client component that use the either the Details EXE or the
//*** DetailsDLL component to demonstrate memory management.
//*******************************************************************
//*******************************************************************

//-------------------------------------------------------------------
//  Required definitions
//-------------------------------------------------------------------
#define UNICODE
#define _WIN32_DCOM

//-------------------------------------------------------------------
//  includes
//-------------------------------------------------------------------
#include <assert.h>
#include <stdio.h>
#include <windows.h>
#include <initguid.h>
#include "..\idl\details.h"

//*******************************************************************
//*******************************************************************
//***  UTILITY FUNCTIONS
//*******************************************************************
//*******************************************************************
void DisplayStatus(wchar_t *pwszMsg, HRESULT hr)
{

    if (hr == S_OK)
    {
        wprintf(TEXT("%s\n"), pwszMsg);
        return;
    }

    if (HRESULT_FACILITY(hr) == FACILITY_WINDOWS)
        hr = HRESULT_CODE(hr);

    wchar_t *pwszStatus;
    FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER |
        FORMAT_MESSAGE_FROM_SYSTEM,
        NULL,
        hr,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPWSTR)&pwszStatus,
        0,
        NULL );

    wprintf(TEXT("%s: %s (ECode: %lx)\n"), pwszMsg, pwszStatus, hr);

    LocalFree(pwszStatus);
}

//*******************************************************************
//*******************************************************************
//*** Main
//*******************************************************************
//*******************************************************************
void main(int argc, char **argv)
{
    //-----------------------------------------
    // This is the main thread
    //-----------------------------------------
    DisplayStatus(TEXT("Client Started"), S_OK);
    HRESULT hr = S_OK;

    hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
    assert(SUCCEEDED(hr));

    IMemoryManagement *pMem=0;
    hr = CoCreateInstance(CLSID_Details, 0, CLSCTX_SERVER,
               IID_IMemoryManagement, reinterpret_cast<void**>(&pMem));

    if (SUCCEEDED(hr)) {

        BYTE *pbImage = new BYTE[10];
        wchar_t *pOutText=0;
        Zone zone; zone.m_sX=50;
        hr = pMem->OcrZoneAndGetImageDimension(10, (byte*)pbImage, &zone, &pOutText);
        wprintf(TEXT("%d\t%s\n"), zone.m_sX, pOutText);
        // new values in zone, pOutText
        delete [] pbImage ;

        Resolution res = { 640, 480 } ;
        Zone zones[5];
        // set the whole array to contain 0's
        memset(zones, 0, sizeof(Zone) * 5 );

        hr = pMem->Pixels2Inches(res, 5, zones);

        // returned array should contain 1's
        for (long i=0; i<5; i++) {
            wprintf(TEXT("[%d,%d,%d,%d] "), zones[i].m_sX, zones[i].m_sY, zones[i].m_sWidth, zones[i].m_sHeight);
        }
        wprintf(TEXT("\n"));

        // simple
        pMem->SetAge(1);
        short age=0; pMem->GetAge(&age);
        age=0; pMem->ChangeAge(&age);

        // demo real reason for CoTaskMem- Alloc, Free, Realloc

        { // [ in ]
            BinaryData data;
            char *pBuffer = "JackAndJill";
            data.m_lSize = strlen(pBuffer);
            // allocate embedded pointer
            // use this
            // data.m_pBuffer = (BYTE*)CoTaskMemAlloc(data.m_lSize);
            // or this
            data.m_pBuffer = new BYTE [data.m_lSize];
            memcpy(data.m_pBuffer, pBuffer, data.m_lSize);
            hr = pMem->SetBinaryData(data);
            DisplayStatus(TEXT("SetBinaryData"), hr);
            // deallocate embedded pointer
            // use this
            //CoTaskMemFree(data.m_pBuffer);
            // or this
            delete [] data.m_pBuffer ;
        }

        { // [ out ]
          BinaryData data;
            hr = pMem->GetBinaryData(&data);
            DisplayStatus(TEXT("GetBinaryData"), hr);
            char szBuf[MAX_PATH]; memset(szBuf, 0, MAX_PATH);
            memcpy(szBuf, data.m_pBuffer, (data.m_lSize>MAX_PATH)?MAX_PATH:data.m_lSize);
            printf("Data=%s\tSize=%d\n", szBuf, data.m_lSize);
            // deallocate embedded pointer
            CoTaskMemFree(data.m_pBuffer);
        }

        { // [ in, out ]
            BinaryData data;
            char *pBuffer = "Humpty";
            data.m_lSize = strlen(pBuffer);
            // allocate embedded pointer
            data.m_pBuffer = (BYTE*)CoTaskMemAlloc(data.m_lSize);
            memcpy(data.m_pBuffer, pBuffer, data.m_lSize);
            hr = pMem->ChangeBinaryData(&data);
            DisplayStatus(TEXT("SetBinaryData"), hr);
            char szBuf[MAX_PATH]; memset(szBuf, 0, MAX_PATH);
            memcpy(szBuf, data.m_pBuffer, (data.m_lSize>MAX_PATH)?MAX_PATH:data.m_lSize);
            printf("Data=%s\tSize=%d\n", szBuf, data.m_lSize);
            // deallocate embedded pointer
            CoTaskMemFree(data.m_pBuffer);
        }

        pMem->Release();
    } else {
        DisplayStatus(TEXT("Failed CoCreateInstance"), hr);
    }

    CoUninitialize();
    DisplayStatus(TEXT("Client shutting down..."), S_OK);
}
