// OCRClientDlg.cpp : implementation file
//

#include "stdafx.h"
#include "OCRClient.h"
#include "OCRClientDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



//*******************************************************************
//*******************************************************************
//*******************************************************************
//****    Binary Data Object 
//*******************************************************************
//*******************************************************************
//*******************************************************************
class BinaryData {
public:
	BinaryData(const CString & strFileName);
	~BinaryData();

	VARIANT & SafeArrayVariant() ;

private:
	BinaryData();
	BinaryData(const BinaryData & rhs);
	BinaryData &operator=(const BinaryData & rhs) ;

protected:
	unsigned char * LoadBinaryData(unsigned long & lSize);
	void ClearBinaryData(unsigned char *pData);
	SAFEARRAY *CreateBinaryDataArray(const unsigned char *pBinaryData, const unsigned long & lSize);
	void DestroyBinaryDataArray(SAFEARRAY *psa) ;

private:
	SAFEARRAY *m_pSafeArrayBuffer;
	VARIANT m_binaryImageArray ;
	CString m_strImageFileLocation ;
};


//*******************************************************************
//*******************************************************************
//*******************************************************************
//****    Binary Data Object Implementation
//*******************************************************************
//*******************************************************************
//*******************************************************************
BinaryData::BinaryData(const CString & strFileName)
	:	m_pSafeArrayBuffer(0),
		m_strImageFileLocation(strFileName),
		m_binaryImageArray()
{
}

BinaryData::~BinaryData()
{
	DestroyBinaryDataArray(m_pSafeArrayBuffer) ;
}

unsigned char * 
BinaryData::LoadBinaryData(unsigned long & lSize)
{
	unsigned char *pData = NULL ;
	lSize = 0 ;
	
	if (m_strImageFileLocation.IsEmpty()) {
		return NULL ;
	}

	CFile f(m_strImageFileLocation, CFile::modeRead);
	lSize = f.GetLength();

	pData = new unsigned char[lSize] ;
	UINT nBytesRead = f.Read(pData, lSize);
	f.Close() ;
	if (lSize!=nBytesRead) {
		throw "unexpected error during file size comparison";
	}
	return pData ;
}

void 
BinaryData::ClearBinaryData(unsigned char *pData)
{
	if (pData) {
		delete [] pData ;
	}
}

SAFEARRAY *
BinaryData::CreateBinaryDataArray(const unsigned char *pBinaryData, const unsigned long & lSize)
{
	SAFEARRAYBOUND sabound[1];
	sabound[0].cElements = lSize;
	sabound[0].lLbound = 0;
	// Create safearray of unsigned char.
	SAFEARRAY *psa = SafeArrayCreate(VT_UI1, 1, sabound);
	if (psa == NULL) AfxThrowMemoryException();		
	// obtain the data pointer
	unsigned char *pc = NULL;   
	SafeArrayAccessData(psa, reinterpret_cast<void HUGEP**>(&pc));  
	// Copy binary data into array    
	memcpy(pc, pBinaryData, lSize);
	SafeArrayUnaccessData(psa);

	return psa ;
}

void 
BinaryData::DestroyBinaryDataArray(SAFEARRAY *psa)
{
	if(psa) {
		SafeArrayDestroy(psa);
	}
}


VARIANT &
BinaryData::SafeArrayVariant()
{	
	unsigned long lSize = 0 ;
	unsigned char *pBinaryData = LoadBinaryData(lSize) ;
	m_pSafeArrayBuffer = CreateBinaryDataArray(pBinaryData, lSize) ;
	ClearBinaryData(pBinaryData) ;
	VariantInit(&m_binaryImageArray) ;
	m_binaryImageArray.vt = (VT_ARRAY | VT_UI1) ;
	m_binaryImageArray.parray = m_pSafeArrayBuffer ;
	return(m_binaryImageArray) ;
}





/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COCRClientDlg dialog

COCRClientDlg::COCRClientDlg(CWnd* pParent /*=NULL*/)
	: CDialog(COCRClientDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(COCRClientDlg)
	m_strServerName = _T("dog");
	m_strImageFileLocation = _T("image.tif");
	m_strOCRResults = _T("");
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void COCRClientDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COCRClientDlg)
	DDX_Text(pDX, IDC_EDIT_SERVER_NAME, m_strServerName);
	DDX_Text(pDX, IDC_EDIT_IMAGE_FILE_LOCATION, m_strImageFileLocation);
	DDX_Text(pDX, IDC_EDIT_OCR_RESULT, m_strOCRResults);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(COCRClientDlg, CDialog)
	//{{AFX_MSG_MAP(COCRClientDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(ID_TYPE_LIB, OnTypeLib)
	ON_BN_CLICKED(ID_COMPILER, OnCompiler)
	ON_BN_CLICKED(ID_MANUAL, OnManual)
	ON_BN_CLICKED(ID_REUSE_MANUAL, OnReuseManual)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COCRClientDlg message handlers

BOOL COCRClientDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void COCRClientDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void COCRClientDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR COCRClientDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

//*******************************************************************
//*******************************************************************
//**  The COleDispatchDriver Technique
//*******************************************************************
//*******************************************************************
#include "ocrserver.h"
void COCRClientDlg::OnTypeLib() 
{
   CWaitCursor wait ;
   // Update the member variables
   UpdateData(TRUE) ;
   // Create the safe array to transfer the binary data 
   BinaryData binaryImageArray(m_strImageFileLocation) ;
   // Use the IOcrProcessor interface
   IOcrProcessorWrapper OcrInterface;
   COleException except;
   if (!OcrInterface.CreateDispatch("OCRServer.OcrProcessor.1", &except)) {
      except.ReportError() ;
      return ;
   }
   // Call interface methods through this wrapper object
   CString strResult = 
       OcrInterface.OcrWholeImage(binaryImageArray.SafeArrayVariant());
   // Done using the interface, so release it
   OcrInterface.ReleaseDispatch();
   // Update the GUI
   m_strOCRResults = strResult ;
   UpdateData(FALSE) ;
}

//*******************************************************************
//*******************************************************************
//**  The #import Technique
//*******************************************************************
//*******************************************************************
#import "..\OCRServer\OCRServer.tlb" 
void COCRClientDlg::OnCompiler() 
{
   CWaitCursor wait ;
   UpdateData(TRUE) ;

   try {
      using namespace OCRSERVERLib ;
      IOcrProcessorPtr pInterface ;
      pInterface = IOcrProcessorPtr("OCRServer.OcrProcessor.1") ;
      BinaryData binaryImageArray(m_strImageFileLocation) ;
      _variant_t binArray(binaryImageArray.SafeArrayVariant());
      _bstr_t bstrResult = pInterface->OcrWholeImage(binArray);
      m_strOCRResults = (char*)bstrResult ;
   } catch (_com_error & error) {
      m_strOCRResults.Format(TEXT("%s[%x]"), error.ErrorMessage(), error.Error());
   }

   UpdateData(FALSE) ;
}

//*******************************************************************
//*******************************************************************
//**  The Manual Technique
//*******************************************************************
//*******************************************************************
#include "..\OCRServer\OCRServer.h"
#include "..\OCRServer\OCRServer_i.c"
#include <atlbase.h>
#include <comdef.h>
void COCRClientDlg::OnManual() 
{
   CWaitCursor wait ;
   // Update the member variables
   UpdateData(TRUE) ;

   // Creating the safe array to transfer the binary data 
   BinaryData binaryImageArray(m_strImageFileLocation) ;

   // Fill in the server info structure
   _bstr_t bstrServerName;
   COSERVERINFO csi = { 0, NULL, NULL, 0 }; 
   // m_strServerName is a COCRClientDlg member variable that 
   // is bounded to the ServerName edit control and it holds 
   // the server name.
   if (!m_strServerName.IsEmpty()) {
      bstrServerName = m_strServerName;
      // Fill in the server name
      csi.pwszName = bstrServerName;
   }

   try {
      // Fill in the multi_qi structure
      MULTI_QI mqi[] = { 
         {&IID_IOcrProcessor, NULL, S_OK}, 
         {&IID_ISpell, NULL, S_OK} 
      };

      HRESULT hr = CoCreateInstanceEx(CLSID_OcrProcessor, NULL, 
         CLSCTX_SERVER, &csi, sizeof(mqi)/sizeof(mqi[0]), mqi);
      // Check for activation errors
      if (FAILED(hr)) _com_issue_error(hr);

      // Check IOcrProcessor
      if (FAILED(mqi[0].hr)) _com_issue_error(mqi[0].hr);
      CComPtr<IOcrProcessor> pOcr = 
         reinterpret_cast<IOcrProcessor*>(mqi[0].pItf);
      reinterpret_cast<IOcrProcessor*>(mqi[0].pItf)->Release();

      BSTR bstrOcrText ;
      hr = pOcr->OcrWholeImage(binaryImageArray.SafeArrayVariant(), &bstrOcrText) ;
      if (FAILED(hr)) _com_issue_error(hr);
      m_strOCRResults = bstrOcrText ;
      ::SysFreeString(bstrOcrText) ;

            // Check ISpell
      if (FAILED(mqi[1].hr)) _com_issue_error(mqi[1].hr);
      CComPtr<ISpell> pSpell = reinterpret_cast<ISpell*>(mqi[1].pItf);
      reinterpret_cast<ISpell*>(mqi[1].pItf)->Release();

      PossibleWords pw;
      hr = pSpell->Check(L"Test", &pw);
      if (FAILED(hr)) _com_issue_error(hr);
      m_strOCRResults += CString(CString("\r\nChoice1:")+CString(pw.wszOne));
      m_strOCRResults += CString(CString("\r\nChoice2:")+CString(pw.wszTwo));

   } catch (_com_error & error) {
      m_strOCRResults.Format(TEXT("%s[%x]"), error.ErrorMessage(), error.Error());
   }

   // Update the GUI
   UpdateData(FALSE) ;

}

//*******************************************************************
//*******************************************************************
//**  The Reusable Technique
//*******************************************************************
//*******************************************************************
#include "..\include\remptr.h"
void COCRClientDlg::OnReuseManual() 
{
   CWaitCursor wait ;
   UpdateData(TRUE) ;

   // Creating the safe array to transfer the binary data 
   BinaryData binaryImageArray(m_strImageFileLocation) ;

   char *pszServerName = m_strServerName.GetBuffer(m_strServerName.GetLength());
   try {
      RemotePtr<IOcrProcessor, &CLSID_OcrProcessor, 
                &IID_IOcrProcessor> pOcr(pszServerName) ;
      BSTR pOcrText ;
      HRESULT hr = pOcr->OcrWholeImage(binaryImageArray.SafeArrayVariant(), 
                                      &pOcrText);
      if (FAILED(hr)) _com_issue_error(hr);
      m_strOCRResults = pOcrText ;
      ::SysFreeString(pOcrText) ;
   } catch (_com_error & error) {
      m_strOCRResults.Format("%s[%x]", error.ErrorMessage(), error.Error());
   }
   m_strServerName.ReleaseBuffer();  
   UpdateData(FALSE) ;
}
