// SecureOcrProcessor.cpp : Implementation of CSecureOcrProcessor
#include "stdafx.h"
#include "SecureOcrServer.h"
#include "SecureOcrProcessor.h"

#include "..\..\include\ImpersonatedUser.cpp"

/////////////////////////////////////////////////////////////////////////////
// CSecureOcrProcessor

STDMETHODIMP CSecureOcrProcessor::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ISecureOcrProcessor
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}


//*******************************************************************
//*  Utility
//*******************************************************************
inline void Chk(HRESULT hr)
{
   if (FAILED(hr)) _com_issue_error(hr);
}

inline long ByteLen(wchar_t *pwsz) 
{
   return (sizeof(wchar_t)*(wcslen(pwsz)+1));
}

//*******************************************************************
//*  Is the user in the hit list?
//*******************************************************************
#define HIT_LIST_NT_GROUP "HitList"  // Ideally, should be a registry entry

bool IsUserInHitList(CImpersonatedUser & user)
{
   // You must add the "HitList" group using "User Manager".
   // Add a user into this group and then test out this functionality.
   string HitListGroup = string(HIT_LIST_NT_GROUP) + string("[") + 
      string(user.ObtainDomainName()) + string("]");

   // Get the group names of all groups that the user belongs to.
   deque<string> *pGroupNames = user.ObtainUserGroupNames() ;
   deque<string>::iterator i ;
   // Loop through the list.
   for (i=pGroupNames->begin(); i!=pGroupNames->end(); i++) {
      string Group = (*i).data();
      if (HitListGroup==Group) {
         // If we get here, the user is int the hit list.
         return true;
      }
   }
   return false;
}


//*******************************************************************
//*  Log security violation
//*******************************************************************
void LogSecurityViolation(CImpersonatedUser & user)
{
   CEventLog EventLog;
   // Because the method call was made without encryption,
   // We must record this violation in the Event Log so that
   // auditors can review it.
   EventLog.LogError(TEXT("Secret has been leaked ")
      TEXT("because a remote invocation was NOT encrypted."), 
      user.ObtainUserSID());
}

//*******************************************************************
//*  Alert the administrator 
//*******************************************************************
#include <lmalert.h>  // needed for NetAlertRaiseEx
#include <lmerrlog.h> 
void GenerateAdminAlert(CImpersonatedUser & user)
{
   TCHAR szExeName[MAX_PATH];
   S_GetExeName(szExeName);

   USES_CONVERSION;
#ifdef UNICODE
   wchar_t *pSvc = szExeName ;
#else
   wchar_t *pSvc = A2W(szExeName) ;
#endif

   wchar_t wszMsg[MAX_PATH];
   wsprintfW(wszMsg, L"THE SYSTEM IS UNDER ATTACK - Hacker: [%s/%s]", 
      A2W(user.ObtainDomainName().data()), 
      A2W(user.ObtainUserName().data()));

   // All OEMs must use NELOG_OEM_Code
   ADMIN_OTHER_INFO ai = {NELOG_OEM_Code, 9};


   int iMsgLen = ByteLen(wszMsg);  // Length of the alert message.
   int NUL_PADDING = 16;   // 8 wide-character are 16 bytes.

   // Total buffer length.
   int iBufLen = sizeof(ai) + iMsgLen + NUL_PADDING;
   BYTE *pAlertBuf = new BYTE[iBufLen];
   memset(pAlertBuf, 0, iBufLen);
   // Copy the header.
   memcpy(pAlertBuf, &ai, sizeof(ai));
   // Skip the place holders for 8 wide-characters.
   BYTE *pMsgOffset = pAlertBuf + sizeof(ai) + NUL_PADDING;
   // Copy the message into the buffer at the appropriate location.
   memcpy(pMsgOffset, wszMsg, iMsgLen);

   // Raise the alert.
   NetAlertRaiseEx(ALERT_ADMIN_EVENT, pAlertBuf, iBufLen, pSvc);
   delete [] pAlertBuf;
}

//*******************************************************************
//*  Validating the call
//*******************************************************************
HRESULT ValidateCall(IServerSecurity *pSecure)
{
   HRESULT hr = S_OK;

   DWORD dwAuthnLevel = 0; void *pClientIdentity = 0;
   // Grab the AuthLevel and ClientIdentity.  We can get the
   // same information using the Access Token (shown shortly).
   hr = pSecure->QueryBlanket(0, 0, 0, &dwAuthnLevel, 0, 
      &pClientIdentity, 0); Chk(hr);
   // Show the client identity so that a user can view it.
   // You should use the MB_SERVICE_NOTIFICATION, in 
   // case you haven't configured
   // RunAs interactive user using dcomcnfg.exe.
   MessageBoxW(NULL, (wchar_t *)pClientIdentity, 
      L"ClientIdentity - Authentication", 
      MB_OK|MB_ICONINFORMATION|MB_SERVICE_NOTIFICATION);

   //---------------------------------------------------
   // Get the client Access Token from the call context
   //---------------------------------------------------
   CImpersonatedUser user;
   user.ObtainAccessToken(pSecure);


   #define E_UNENCRYPTED_CALL 0x201
   #define E_IN_HIT_LIST      0x202
   //---------------------------------------------------
   // Audit Trail: Call must be made using RPC_C_AUTHN_LEVEL_PKT_PRIVACY
   // or we will record this violation in the audit trail.
   //---------------------------------------------------
   if (dwAuthnLevel!=RPC_C_AUTHN_LEVEL_PKT_PRIVACY) {
      LogSecurityViolation(user);
      hr = AtlReportError(CLSID_SecureOcrProcessor, 
         TEXT("[Security Violation] Secret has been leaked ")
         TEXT("because a remote invocation was NOT encrypted."),
         IID_ISecureOcrProcessor, 
         MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, E_UNENCRYPTED_CALL));
      return hr;
   }


   //---------------------------------------------------
   // Administrative Alert
   //---------------------------------------------------
   if (IsUserInHitList(user)) {
      GenerateAdminAlert(user);
      TCHAR szMsg[MAX_PATH];
      wsprintf(szMsg, 
         TEXT("[Security Violation] Be careful.")
         TEXT("You [%s/%s] are in our hit list."),
         user.ObtainDomainName().data(), 
         user.ObtainUserName().data());
      hr = AtlReportError(CLSID_SecureOcrProcessor, szMsg,
         IID_ISecureOcrProcessor, 
         MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, E_IN_HIT_LIST));
      return hr;
   }

   return hr;
}

//*******************************************************************
//*  A method that performs custom security checks.
//*******************************************************************
STDMETHODIMP CSecureOcrProcessor::SecureOcrImage()
{
   HRESULT hr = S_OK;
   try {
      CComPtr<IServerSecurity> pSecure;
      hr = CoGetCallContext(IID_IServerSecurity, 
         reinterpret_cast<void**>(&pSecure)); Chk(hr);

      if ( (hr=ValidateCall(pSecure)) == S_OK ) {
         hr = pSecure->ImpersonateClient(); Chk(hr);
         //**********************************************
         //* Here's where you would do work for the client
         //* using the client's credentials.
         //**********************************************
         hr = pSecure->RevertToSelf(); Chk(hr);
      }

   } catch (_com_error & er) {
      MessageBox(NULL, er.ErrorMessage(), TEXT("ErrorMessage"), 
         MB_OK|MB_ICONINFORMATION|MB_SERVICE_NOTIFICATION);
      hr = er.Error();
   }
   return hr;
}
